/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


#ifndef __TOOLBOX_HASHSET_H
#define __TOOLBOX_HASHSET_H


namespace toolbox
{
	/**
	 * A hash set stores any kind of object with
	 * nearly constant save/lookup time, but with
	 * consuming about 20 times the memory of an array.
	 * The main and only difference to the hashtable is
	 * that it stores the keys only, not representing
	 * a key/value, but a 'key exist?' lookup.
	 * @see See collections.txt for an overview of the
	 * available collection classes.
	 * @author Thomas Jacob
	 */
	class HashSet : protected HashMap<Void>
	{
		public:

			/**
			 * Creates a new hash set.
			 */
			HashSet();

			/**
			 * Creates a new hash set with an initial size.
			 * @param initSize The minimum and initial size of the hashtable.
			 * This is the minimum and initial number of buckets.
			 */
			HashSet(long initSize);

			/**
			 * Creates a new hash set with customized settings.
			 * @param initSize The minimum and initial size of the hashtable.
			 * This is the minimum and initial number of buckets.
			 * @param desiredFillRatio The fill ratio after a hashtable resize.
			 * This happens, if the fill ratio gets higher
			 * than one and a half times the fill ratio
			 * or lower than half the fill ratio.
			 * The rest of the buckets is reserved for
			 * further setting of keys.
			 */
			HashSet(long initSize, int desiredFillRatio);

			/**
			 * Destroys the hash set. The memory of the
			 * keys is not freed, since it was not acquired,
			 */
			virtual ~HashSet();

			/**
			 * Adds a key to the hash set.
			 * Afterwards, the method checks if the hash set
			 * has to be resized.
			 * @param key The key to be set. Since the memory
			 * of the key is not freed after unsetting the key
			 * or deleting the hash set, the memory management
			 * is still yours after the call.
			 * Note that NULL is a valid value among all possible values for keys.
			 */
			inline void Add(void * key);

			/**
			 * Returns, if the hash set contains a given key.
			 * @param key The key to be looked up.
			 * Note that NULL is a valid value among all possible values for keys.
			 * @return if the hash set contains the key.
			 */
			inline bool Contains(void * key) const;

			/**
			 * Returns a new PointeredList of all
			 * keys currently in the hash set.
			 * @return The keys as a PointeredList.
			 * @warning This method creates a PointeredList
			 * and expects that it is deleted by the caller.
			 */
			inline PointeredList * GetAll() const;

			/**
			 * Returns the number of added elements.
			 * @return The number of added elements.
			 */
			inline long GetCount() const;

			/**
			 * Returns whether there are no elements in the hash set.
			 * @return Whether there are no elements in the hash set.
			 */
			inline bool IsEmpty() const;

			/**
			 * Removes a key from the hash set. Afterwards,
			 * the method checks if the hash set has to be resized.
			 * The memory of the key is not freed.
			 * @param key The key to be looked up for removal.
			 * Note that NULL is a valid value among all possible values for keys.
			 */
			inline void Remove(void * key);

			/**
			 * Removes all keys from the hash set. Afterwards,
			 * the method checks if the hash set has to be resized.
			 * The memory of the key is not freed.
			 */
			inline void RemoveAll();

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * An IntKeyHashSet stores objects in the same way
	 * as the HashSet does, but uses long ints as keys.
	 * @author Thomas Jacob
	 */
	class IntKeyHashSet : public IntKeyHashMap<Void>
	{
		public:

			/**
			 * Creates a new int key hash set.
			 */
			IntKeyHashSet();

			/**
			 * Creates a new int key hash set with an initial size.
			 * @param initSize The minimum and initial size of the hashtable.
			 * This is the minimum and initial number of buckets.
			 */
			IntKeyHashSet(long initSize);

			/**
			 * Creates a new int key hash set with customized settings.
			 * @param initSize The minimum and initial size of the hashtable.
			 * This is the minimum and initial number of buckets.
			 * @param desiredFillRatio The fill ratio after a hashtable resize.
			 * This happens, if the fill ratio gets higher
			 * than one and a half times the fill ratio
			 * or lower than half the fill ratio.
			 * The rest of the buckets is reserved for
			 * further setting of keys.
			 */
			IntKeyHashSet(long initSize, int desiredFillRatio);

			/**
			 * Destroys the int key hash set. The memory of the
			 * keys is not freed, since it was not acquired,
			 */
			virtual ~IntKeyHashSet();

			/**
			 * Adds a key to the hash set.
			 * Afterwards, the method checks if the hash set
			 * has to be resized.
			 * @param key The key to be set. Since the memory
			 * of the key is not freed after unsetting the key
			 * or deleting the hash set, the memory management
			 * is still yours after the call.
			 */
			inline void Add(long key);

			/**
			 * Returns, if the hash set contains a given key.
			 * @param key The key to be looked up.
			 * @return if the hash set contains the key.
			 */
			inline bool Contains(long key) const;

			/**
			 * Returns a new PointeredList of all
			 * keys currently in the hash set.
			 * @return The keys as a PointeredList.
			 * @warning This method creates a PointeredList
			 * and expects that it is deleted by the caller.
			 */
			inline PointeredList * GetAll() const;

			/**
			 * Removes a key from the hash set. Afterwards,
			 * the method checks if the hash set has to be resized.
			 * The memory of the key is not freed.
			 * @param key The key to be looked up for removal.
			 */
			inline void Remove(long key);

			/**
			 * Removes all keys from the hash set. Afterwards,
			 * the method checks if the hash set has to be resized.
			 * The memory of the key is not freed.
			 */
			inline void RemoveAll();

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * A StringKeyHashSet stores objects in the same way
	 * as the HashSet does, but uses char arrays (strings)
	 * as keys. To keys equal each other, if they contain
	 * the same characters.
	 * @author Thomas Jacob
	 */
	class StringKeyHashSet : public StringKeyHashMap<Void>
	{
		public:

			/**
			 * Creates a new string key hash set.
			 */
			StringKeyHashSet();

			/**
			 * Creates a new string key hash set with an initial size.
			 * @param initSize The minimum and initial size of the hashtable.
			 * This is the minimum and initial number of buckets.
			 */
			StringKeyHashSet(long initSize);

			/**
			 * Creates a new string key hash set with customized settings.
			 * @param initSize The minimum and initial size of the hashtable.
			 * This is the minimum and initial number of buckets.
			 * @param desiredFillRatio The fill ratio after a hashtable resize.
			 * This happens, if the fill ratio gets higher
			 * than one and a half times the fill ratio
			 * or lower than half the fill ratio.
			 * The rest of the buckets is reserved for
			 * further setting of keys.
			 */
			StringKeyHashSet(long initSize, int desiredFillRatio);

			/**
			 * Destroys the string key hash set. The memory of the
			 * keys is not freed, since it was not acquired,
			 */
			virtual ~StringKeyHashSet();

			/**
			 * Adds a key to the hash set.
			 * Afterwards, the method checks if the hash set
			 * has to be resized.
			 * @param key The key to be set. Since the memory
			 * of the key is not freed after unsetting the key
			 * or deleting the hash set, the memory management
			 * is still yours after the call.
			 */
			inline void Add(const char * key);

			/**
			 * Returns, if the hash set contains a given key.
			 * @param key The key to be looked up.
			 * @return if the hash set contains the key.
			 */
			inline bool Contains(const char * key) const;

			/**
			 * Returns a new PointeredList of all
			 * keys currently in the hash set.
			 * @return The keys as a PointeredList.
			 * @warning This method creates a PointeredList
			 * and expects that it is deleted by the caller.
			 */
			inline PointeredList * GetAll() const;

			/**
			 * Removes a key from the hash set. Afterwards,
			 * the method checks if the hash set has to be resized.
			 * The memory of the key is not freed.
			 * @param key The key to be looked up for removal.
			 */
			inline void Remove(const char * key);

			/**
			 * Removes all keys from the hash set. Afterwards,
			 * the method checks if the hash set has to be resized.
			 * The memory of the key is not freed.
			 */
			inline void RemoveAll();

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * An ICStringKeyHashSet stores objects in the same way
	 * as the StringKeyHashSet does, but the comparisons are case insensitive.
	 * @author Thomas Jacob
	 */
	class ICStringKeyHashSet : public ICStringKeyHashMap<Void>
	{
		public:

			/**
			 * Creates a new ignore case string key hash set.
			 */
			ICStringKeyHashSet();

			/**
			 * Creates a new ignore case string key hash set with an initial size.
			 * @param initSize The minimum and initial size of the hashtable.
			 * This is the minimum and initial number of buckets.
			 */
			ICStringKeyHashSet(long initSize);

			/**
			 * Creates a new ignore case string key hash set with customized settings.
			 * @param initSize The minimum and initial size of the hashtable.
			 * This is the minimum and initial number of buckets.
			 * @param desiredFillRatio The fill ratio after a hashtable resize.
			 * This happens, if the fill ratio gets higher
			 * than one and a half times the fill ratio
			 * or lower than half the fill ratio.
			 * The rest of the buckets is reserved for
			 * further setting of keys.
			 */
			ICStringKeyHashSet(long initSize, int desiredFillRatio);

			/**
			 * Destroys the ignore case string key hash set. The memory of the
			 * keys is not freed, since it was not acquired,
			 */
			virtual ~ICStringKeyHashSet();

			/**
			 * Adds a key to the hash set.
			 * Afterwards, the method checks if the hash set
			 * has to be resized.
			 * @param key The key to be set. Since the memory
			 * of the key is not freed after unsetting the key
			 * or deleting the hash set, the memory management
			 * is still yours after the call.
			 */
			inline void Add(const char * key);

			/**
			 * Returns, if the hash set contains a given key.
			 * @param key The key to be looked up.
			 * @return if the hash set contains the key.
			 */
			inline bool Contains(const char * key) const;

			/**
			 * Returns a new PointeredList of all
			 * keys currently in the hash set.
			 * @return The keys as a PointeredList.
			 * @warning This method creates a PointeredList
			 * and expects that it is deleted by the caller.
			 */
			inline PointeredList * GetAll() const;

			/**
			 * Removes a key from the hash set. Afterwards,
			 * the method checks if the hash set has to be resized.
			 * The memory of the key is not freed.
			 * @param key The key to be looked up for removal.
			 */
			inline void Remove(const char * key);

			/**
			 * Removes all keys from the hash set. Afterwards,
			 * the method checks if the hash set has to be resized.
			 * The memory of the key is not freed.
			 */
			inline void RemoveAll();

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};
}


#endif
