/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


#ifndef __TOOLBOX_REFERENCE_H
#define __TOOLBOX_REFERENCE_H


namespace toolbox
{
	#if defined(_DEBUG) && (defined(_AFX) || defined(_AFXDLL))
		class AutoDeleting : public CObject
	#else
		class AutoDeleting
	#endif
	{
		friend Reference;

		private:

			ArrayList<Reference> References;

			void AddReference(class Reference * Reference);

			void RemoveReference(class Reference * Reference);

		public:

			virtual ~AutoDeleting();

			inline const ArrayList<Reference> & GetReferences() const;
	};

	#if defined(_DEBUG) && (defined(_AFX) || defined(_AFXDLL))
		class Reference : public CObject
	#else
		class Reference
	#endif
	{
		friend AutoDeleting;

		private:

			void * Referrer;

		protected:

			Reference(void * referrer);

			virtual void AddElement(AutoDeleting * Element) = NULL;

			virtual ArrayList<AutoDeleting> * GetElements() const = NULL;

			virtual void RemoveElement(AutoDeleting * Element) = NULL;

		public:

			virtual ~Reference();

			inline void * GetReferrer() const;
	};

	template <class ELEMENTTYPE> class SingleReference : public Reference
	{
		private:

			class ELEMENTTYPE * Element;

			static const AutoDeleting * ParameterConstraint;

		protected:

			virtual void AddElement(AutoDeleting * Element);

			virtual ArrayList<AutoDeleting> * GetElements() const;

			virtual void RemoveElement(AutoDeleting * Element);

		public:

			SingleReference(void * Referrer);

			virtual ~SingleReference();

			inline ELEMENTTYPE * GetElement() const;

			void SetElement(ELEMENTTYPE * Element);

			inline void UnsetElement();
	};

	template <class ELEMENTTYPE> class ReferenceList : public Reference
	{
		private:

			ArrayList<ELEMENTTYPE> * Elements;

			static const AutoDeleting * ParameterConstraint;

		protected:

			virtual void AddElement(AutoDeleting * Element);

			virtual ArrayList<AutoDeleting> * GetElements() const;

			virtual void RemoveElement(AutoDeleting * Element);

		public:

			/**
			 * Creates a new array.
			 */
			ReferenceList();

			/**
			 * Creates a new array.
			 * @param initSize The minimum and initial size of the array.
			 */
			ReferenceList(long initSize);

			/**
			 * Creates a new array.
			 * @param initSize The minimum and initial size of the array.
			 * @param minimumFillRatio The minimum fill ratio for the array.
			 * @param desiredFillRatio The fill ratio after an array resize.
			 */
			ReferenceList(long initSize, int minimumFillRatio, int desiredFillRatio);

			/**
			 * Destroys the array. When deleting the array,
			 * all contained elements are also freed.
			 */
			~ReferenceList();

			/**
			 * Appends an element to the array.
			 * @param Element The element to be appended.
			 */
			void Append(ELEMENTTYPE * Element);

			/**
			 * Appends all elements of another array list to this one.
			 * @warning The elements' memory is shared by both array lists,
			 * so one array should get its elements unlinked before destruction.
			 * @param Elements The elements to be appended.
			 */
			void AppendAll(const ArrayList<ELEMENTTYPE> * Elements);

			/**
			 * Returns whether a given element is contained in this array.
			 * This method uses the Compare method to compare the elements.
			 * @note This method performs a linear search.
			 * @param Element The element to be searched.
			 * @return Whether the element is contained.
			 */
			bool Contains(const ELEMENTTYPE * Element) const;

			/**
			 * Returns whether a given element is contained in this array.
			 * This method uses a given compare method to compare the elements.
			 * @note This method performs a linear search.
			 * @param Element The element to be searched.
			 * @param compare A functions that compares two elements and
			 * returns -1, if the first is less than the second, 0, if both
			 * elements are equal and 1, if the first is more than the second.
			 * @return Whether the element is contained.
			 */
			bool Contains(const ELEMENTTYPE * Element,
				int (* compare)(const ELEMENTTYPE * Element1, const ELEMENTTYPE * Element2)) const;

			/**
			 * Returns the 0-indexed position of a given element within the array list.
			 * This method uses the Compare method to compare the elements.
			 * @note This method performs a linear search.
			 * @param element The element to be looked up.
			 * @return The 0-indexed position of the element or -1,
			 * if the element is not found.
			 */
			int Find(const ELEMENTTYPE * Element) const;

			/**
			 * Returns the 0-indexed position of a given element within the array list.
			 * This method uses a given compare method to compare the elements.
			 * @note This method performs a linear search.
			 * @param element The element to be looked up.
			 * @param compare A functions that compares two elements and
			 * returns -1, if the first is less than the second, 0, if both
			 * elements are equal and 1, if the first is more than the second.
			 * @return The 0-indexed position of the element or -1,
			 * if the element is not found.
			 */
			int Find(const ELEMENTTYPE * Element,
				int (* compare)(const ELEMENTTYPE * Element1, const ELEMENTTYPE * Element2)) const;

			/**
			 * Returns the element at a given position.
			 * @param nr The position of the element. If nr is invalid, the method returns NULL.
			 * @return The element at the position or NULL.
			 */
			ELEMENTTYPE * Get(long nr) const;

			/**
			 * Returns the number of used array cells.
			 * This is the number of added objects.
			 * @return The number of used array cells.
			 */
			inline long GetCount() const;

			/**
			 * Returns the fill ratio after an array resize.
			 * @return The fill ratio after an array resize.
			 * @see DesiredFillRatio
			 */
			inline int GetDesiredFillRatio() const;

			/**
			 * Returns the current fill ratio.
			 * @return The current fill ratio.
			 */
			inline int GetFillRatio() const;

			/**
			 * Returns the first element of the array.
			 * If the array is empty, the method returns NULL.
			 * @return The first element or NULL.
			 */
			ELEMENTTYPE * GetFirst() const;

			/**
			 * Returns the last element of the array.
			 * If the array is empty, the method returns NULL.
			 * @return The last element or NULL.
			 */
			ELEMENTTYPE * GetLast() const;

			/**
			 * Returns the minimum fill ratio for the array.
			 * @return The minimum fill ratio for the array.
			 * @see MinimumFillRatio
			 */
			inline int GetMinimumFillRatio() const;

			/**
			 * Inserts an element before a given position.
			 * All elements at and after the position are moved by one.
			 * @param Element The element to be inserted.
			 * @param nr The position to insert before. If nr is invalid, nothing happens.
			 */
			void Insert(const ELEMENTTYPE * Element, long nr);

			/**
			 * Inserts all elements of another array list before a given position.
			 * All elements at and after the position are moved.
			 * @warning The elements' memory is shared by both array lists,
			 * so one array should get its elements unlinked before destruction.
			 * @param Elements The elements to be inserted.
			 * @param nr The position to insert before. If nr is invalid, nothing happens.
			 */
			void InsertAll(const ArrayList<ELEMENTTYPE> * Elements, long nr);

			/**
			 * Returns, whether there are no elements in the array.
			 */
			inline bool IsEmpty() const;

			/**
			 * Prepends an element to the array.
			 * @param Element The element to be prepended.
			 */
			inline void Prepend(const ELEMENTTYPE * Element);

			/**
			 * Prepends all elements of another array list to this one.
			 * @warning The elements' memory is shared by both array lists,
			 * so one array should get its elements unlinked before destruction.
			 * @param Elements The elements to be prepended.
			 */
			inline void PrependAll(const ArrayList<ELEMENTTYPE> * Elements);

			/**
			 * Replaces an element by another one.
			 * The replaced element is not freed, but returned.
			 * @param NewElement The element that replaces the old one.
			 * @param nr The position of the element to be replaced.
			 * If nr is invalid, nothing happens and the method returns NULL.
			 * @return The old element.
			 */
			ELEMENTTYPE * Replace(ELEMENTTYPE * NewElement, long nr);

			/**
			 * Deletes an element at a given position.
			 * The element is removed from the array and freed.
			 * @param nr The position of the element. If nr is invalid, nothing happens.
			 * @return Whether the element was found.
			 */
			bool Remove(long nr);

			/**
			 * Deletes multiple elements at a given range.
			 * The elements are removed from the array and freed.
			 * @param from The starting position of the range.
			 * If from is invalid, nothing happens.
			 * @param count The number of elements to be removed.
			 * If count is invalid, nothing happens.
			 * @return Whether the range was found.
			 */
			bool Remove(long from, long count);

			/**
			 * Deletes a given element from the array.
			 * The element is searched, removed from the array and freed.
			 * If the element is not contained, nothing happens.
			 * If the element is contained multiple times, only one instance is deleted.
			 * @param Element The element to be deleted.
			 * @return Whether the element was found.
			 */
			bool Remove(ELEMENTTYPE * Element);

			/**
			 * Deletes all elements from the array.
			 * All elements are removed from the array and freed.
			 */
			void RemoveAll();

			/**
			 * Deletes the first element from the array.
			 * The element is removed from the array and freed.
			 * If the array is empty, nothing happens.
			 * @return Whether the element was found.
			 */
			inline bool RemoveFirst();

			/**
			 * Deletes the last element from the array.
			 * The element is removed from the array and freed.
			 * If the array is empty, nothing happens.
			 * @return Whether the element was found.
			 */
			bool RemoveLast();

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif

			/**
			 * Shuffles all entries randomly.
			 */
			void Shuffle();

			/**
			 * Sorts the elements in the array according to
			 * the Compare method. The whole array has a sorted
			 * order afterwards, but is not harmed at all
			 * in any other way. All elements remain the same.
			 */
			inline void Sort();

			/**
			 * Sorts the elements in the array according to
			 * a given comparison function. The whole array
			 * has a sorted order afterwards, but is not
			 * harmed at all in any other way. All elements
			 * remain the same.
			 * @param compare A functions that compares two elements and
			 * returns -1, if the first is less than the second, 0, if both
			 * elements are equal and 1, if the first is more than the second.
			 */
			void Sort(int (* compare)(const ELEMENTTYPE * Element1, const ELEMENTTYPE * Element2));
	};

	template <class VALUETYPE> class ReferenceValueMap : Reference
	{
		protected:

			virtual void AddElement(VALUETYPE * Element);

			virtual void RemoveElement(VALUETYPE * Element);

		public:

			virtual ArrayList<AutoDeleting> * GetElements() const;
	};

	template <class VALUETYPE> class IntKeyReferenceValueMap : Reference
	{
		protected:

			virtual void AddElement(VALUETYPE * Element);

			virtual void RemoveElement(VALUETYPE * Element);

		public:

			virtual ArrayList<AutoDeleting> * GetElements() const;
	};

	template <class VALUETYPE> class StringKeyReferenceValueMap : Reference
	{
		protected:

			virtual void AddElement(VALUETYPE * Element);

			virtual void RemoveElement(VALUETYPE * Element);

		public:

			virtual ArrayList<AutoDeleting> * GetElements() const;
	};

	template <class VALUETYPE> class ICStringKeyReferenceValueMap : Reference
	{
		protected:

			virtual void AddElement(VALUETYPE * Element);

			virtual void RemoveElement(VALUETYPE * Element);

		public:

			virtual ArrayList<AutoDeleting> * GetElements() const;
	};

	template <class KEYTYPE, class VALUETYPE> class ReferenceKeyMap : Reference
	{
		protected:

			virtual void AddElement(KEYTYPE * Element);

			virtual void RemoveElement(KEYTYPE * Element);

		public:

			virtual ArrayList<AutoDeleting> * GetElements() const;
	};
}


#endif
