/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


#ifndef __TOOLBOX_SIMULATION_H
#define __TOOLBOX_SIMULATION_H


namespace toolbox
{
	/**
	 * The Simulation class represents an event based
	 * simulation. In consists of a simulation clock,
	 * a array of all objects of the simulation, and a
	 * arrray of all events in the future. The clock is
	 * advanced from event to event, and for each event,
	 * an event method is called, which adds new events,
	 * removes existing ones, modifies objects, creates
	 * or destroys them, or ends the simulation.
	 * @author Thomas Jacob
	 */
	class Simulation
	{
		private:

			/**
			 * The simulation clock. This is the time of the current
			 * event being handled and is advanced in irregular steps
			 * according to the occurring events of the simulation.
			 */
			long Clock;

			/**
			 * The array of the simulation events in the future.
			 */
			ArrayList<SimulationEvent> * Events;

			/**
			 * If the simulation has ended.
			 */
			bool HasEndedFlag;

			/**
			 * The array of the simulation objects.
			 */
			ArrayList<SimulationObject> * Objects;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		public:

			/**
			 * Creates a new simulation.
			 */
			Simulation();

			/**
			 * Destroys the simulation.
			 */
			~Simulation();

			/**
			 * Adds an event to the simulation.
			 * @param Event The event to be added.
			 * @param time The time to schedule the event to.
			 */
			void AddEvent(SimulationEvent * Event, long time);

			/**
			 * Adds an object to the simulation.
			 * @param Object The object to be added.
			 */
			void AddObject(SimulationObject * Object);

			/**
			 * Deletes an event of the simulation.
			 * The event is looked up, removed and freed.
			 * @param eventNr The position of the event in the event array.
			 */
			void DeleteEvent(int eventNr);

			/**
			 * Deletes an event of the simulation.
			 * The event is looked up, removed and freed.
			 * @param Event The event to be deleted.
			 */
			void DeleteEvent(SimulationEvent * Event);

			/**
			 * Deletes an object of the simulation.
			 * The object is looked up, removed and freed.
			 * @param objectNr The position of the object in the object array.
			 */
			void DeleteObject(int objectNr);

			/**
			 * Deletes an object of the simulation.
			 * The object is looked up, removed and freed.
			 * @param Object The object to be deleted.
			 */
			void DeleteObject(SimulationObject * Object);

			/**
			 * Ends the simulation. Use this method in an
			 * event method to notify the simulation to stop
			 * after the current event has been handled.
			 */
			inline void End();

			/**
			 * Returns the simulation clock.
			 * This is the time of the current event.
			 * @return The simulation clock.
			 */
			inline long GetClock();

			/**
			 * Returns an event of the event array
			 * at a given position.
			 * @param eventNr The position of the event.
			 * @return The event at that position.
			 */
			SimulationEvent * GetEvent(int eventNr);

			/**
			 * Returns the number of events in the event array.
			 * @return The number of events in the event array.
			 */
			inline long GetEventCount();

			/**
			 * Returns an object of the object array
			 * at a given position.
			 * @param objectNr The position of the object.
			 * @return The object at that position.
			 */
			SimulationObject * GetObject(int objectNr);

			/**
			 * Returns the number of objects in the object array.
			 * @return The number of objects in the object array.
			 */
			inline long GetObjectCount();

			/**
			 * Returns, if the simulation has ended.
			 * @return If the simulation has ended.
			 */
			inline bool HasEnded();

			/**
			 * Resets the simulation. All events and objects
			 * are removed from their respective arrays.
			 */
			void Reset();

			/**
			 * Starts the simulation. The method takes an event
			 * of the event array after another and executes its
			 * event method (Handle). Events are automatically
			 * deleted after they have been executed. The method
			 * returns, when the End method has been called to
			 * indicate the end of the simulation.
			 */
			void Run();

			/**
			 * Starts the simulation. The method takes an event
			 * of the event array after another and executes its
			 * event method (Handle). Events are automatically
			 * deleted after they have been executed. The method
			 * returns, when the End method has been called to
			 * indicate the end of the simulation, or if a given
			 * time has passed. The method does this by adding a
			 * SimulationEndEvent to the event array before the
			 * simulation is started, and then uses the Run()
			 * overload.
			 * @param endTime The time to end the simulation,
			 * if it is not stopped before.
			 */
			void Run(long endTime);

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * A SimulationEvent changes the simulation state
	 * at a given time. This includes object and/or event
	 * creation, modification and/or deletion. Events are
	 * sorted along a time axis, and the simulation clock
	 * jumps from event to event, while the simulation
	 * executes each event's method (Handle).
	 * @author Thomas Jacob
	 */
	class SimulationEvent
	{
		friend Simulation;

		private:

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

			/**
			 * The corresponding simulation for the event.
			 */
			class Simulation * Simulation;

			/**
			 * The time when the event shall happen.
			 */
			long Time;

		public:

			/**
			 * Creates a new simulation event.
			 */
			SimulationEvent();

			/**
			 * Destroys the simulation event.
			 */
			virtual ~SimulationEvent();

			/**
			 * Returns the corresponding simulation for the event.
			 * @return The corresponding simulation for the event.
			 */
			inline class Simulation * GetSimulation();

			/**
			 * Returns the time when the event shall happen.
			 * @return The time when the event shall happen.
			 */
			inline long GetTime();

			/**
			 * The main event method. This method is called by the
			 * simulation to perform the specific event actions, such
			 * as event or object creation, modification or deletion.
			 * Override this method in your derived events.
			 * Events are automatically deleted after they have been executed.
			 */
			virtual void Handle();

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * The SimulationEndEvent is the simulation event
	 * that the simulation shall stop handling events.
	 * It uses the Simulation::End method to do this.
	 * @author Thomas Jacob
	 */
	class SimulationEndEvent : public SimulationEvent
	{
		private:

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		public:

			/**
			 * Creates a new simulation end event.
			 */
			SimulationEndEvent();

			/**
			 * Destroys the simulation end event.
			 */
			virtual ~SimulationEndEvent();

			/**
			 * Handles the end event by calling
			 * the simulation's end method.
			 */
			virtual void Handle();

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * A SimulationObject is a part of the modelled world,
	 * that shall be simulated. Derive from this class to
	 * define your world's objects and add them to the simulation.
	 * @author Thomas Jacob
	 */
	class SimulationObject
	{
		friend Simulation;

		private:

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

			/**
			 * The corresponding simulation for the object.
			 */
			class Simulation * Simulation;

		public:

			/**
			 * Creates a new simulation object.
			 */
			SimulationObject();

			/**
			 * Destroys the simulation object.
			 */
			virtual ~SimulationObject();

			/**
			 * Returns the corresponding simulation for the object.
			 * @return The corresponding simulation for the object.
			 */
			inline class Simulation * GetSimulation();

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};
}


#endif
